import java.awt.Color;
import java.awt.Container;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;

/**
 * Klasse for grafisk brukergrensesnitt til spillet Fire-p-rad.
 * Klassen viser hovedvinduet til spillet
 * @author Trond Hellem B
 * modifisert av Khalid Azim Mughal
 */
public class GUIHoved extends JFrame implements ActionListener{

  private JPanel venstrePanel;//Panel for venstre halvdel av hovedvinduet
  private JPanel hyrePanel;  //Panel for hyre halvdel av hovedvinduet

  //Knapper for valg
  private JButton nyttSpill, pneSpill,avslutt;
  private JButton spiller1GjrTrekk, spiller2GjrTrekk;

  //Tekstfelt til  fylle inn spillertrekk og til  gi beskjeder
  private JTextField spiller1Trekk, spiller2Trekk, beskjeder;

  //referanse til Spill-objektet som inneholder all informasjonen om
  //spillets gang
  private ISpill spillet;

  /**
   * Kontruktr. Oppretter et nytt hovedvindu og for spillet Fire-P-Rad
   * Initialiserer utseendet til vinduet
   */
  GUIHoved(){
    super("Fire p rad - Hovedvindu");
    Container content=getContentPane();

    //velger  dele hovedvinduet i 2 deler hyre og venstre
    content.setLayout(new GridLayout(1,2));

    //panel som skal inneholde alt i hyre halvdel av vinduet
    hyrePanel=new JPanel(new GridLayout(4,1));

    //oppretter knapper for valg
    nyttSpill=new JButton("Nytt spill");
    pneSpill=new JButton("pne lagret spill");
    avslutt=new JButton("Avslutt");

    //setter dette vinsuet som lytter for knappene
    nyttSpill.addActionListener(this);
    pneSpill.addActionListener(this);
    avslutt.addActionListener(this);

    JPanel knappeValg=new JPanel(new FlowLayout());
    knappeValg.add(nyttSpill);
    knappeValg.add(pneSpill);
    knappeValg.add(avslutt);
    knappeValg.setBackground(Color.white);

    //komponenter for spiller 1 valg av trekk
    JPanel spiller1Valg=new JPanel();
    spiller1GjrTrekk=new JButton("Spiller 1 trekk");
    spiller1GjrTrekk.addActionListener(this);
    spiller1Trekk=new JTextField("0");
    spiller1GjrTrekk.setEnabled(false);
    spiller1Trekk.setEditable(false);
    spiller1Trekk.setColumns(3);
    spiller1Valg.add(spiller1GjrTrekk);
    spiller1Valg.add(spiller1Trekk);
    spiller1Valg.setBackground(Color.white);

    //komponenter for spiller 2 valg av trekk
    JPanel spiller2Valg=new JPanel();
    spiller2GjrTrekk=new JButton("Spiller 2 trekk");
    spiller2GjrTrekk.addActionListener(this);
    spiller2Trekk=new JTextField("0");
    spiller2GjrTrekk.setEnabled(false);
    spiller2Trekk.setEditable(false);
    spiller2Trekk.setColumns(3);
    spiller2Valg.add(spiller2GjrTrekk);
    spiller2Valg.add(spiller2Trekk);
    spiller2Valg.setBackground(Color.white);

    JPanel spillerValg=new JPanel(new GridLayout(2,1));
    spillerValg.add(spiller1Valg);
    spillerValg.add(spiller2Valg);
    spillerValg.setBackground(Color.white);

    //Oppratter et tekstfelt for beskjeder til brukeren
    beskjeder=new JTextField("");
    beskjeder.setEditable(false);
    beskjeder.setColumns(20);
    JPanel beskjedPanel=new JPanel();
    beskjedPanel.add(beskjeder);
    beskjedPanel.setBackground(Color.white);

    //Legger til de forskjellige komponentene til hyre halvdel
    hyrePanel.add(knappeValg);
    hyrePanel.add(spillerValg);
    hyrePanel.add(beskjedPanel);
    hyrePanel.setBackground(Color.white);

    //Venstre halvdel settes til et tomt panel, skal senere inneholde et
    //spillebrett
    venstrePanel=new JPanel();

    //legger det hele til i hovedvinduet
    content.add(venstrePanel);
    content.add(hyrePanel);

    pack();
    setVisible(true);
  }

  /**
   * Metode for  fange opp knappetrykk-hendelser
   * @param e Hendelsen som setter i gang denne metoden
   */
  public void actionPerformed(ActionEvent e){
    if(e.getSource()==nyttSpill){
      //Vis nytt spill dialog
      new GUINyttSpill(this);
    }
    else if(e.getSource()==pneSpill){
      //Be bruker om filnavn vi skal lese fra
      String filnavn=JOptionPane.showInputDialog(this,
           "Skriv inn filnavnet som spillet er lagret som");
      if(filnavn!=null)
        //prv  lese inn spillet
        lesSpillFraFil(filnavn);
    }
    else if(e.getSource()==spiller1GjrTrekk){
      //spiller 1 prver et trekk
      prvTrekk(1);
    }
    else if(e.getSource()==spiller2GjrTrekk){
      //spiller 2 prver et trekk
      prvTrekk(2);
    }
    else if(e.getSource()==avslutt){
      //sjekk om spillet er ferdig. Dersom ikke: lagre til fil
      if(spillet!=null && !spillet.spillErFerdig()){
        String filnavn=JOptionPane.showInputDialog(this,
               "Spillet er ikke ferdig." +
               "\nSkriv inn et filnavn som spillet skal lagres til.");
        if(filnavn!=null)
          //skriv spillet til fil med det gitte filnavnet
          skrivSpillTilFil(filnavn);
      }
      setVisible(false);
      System.exit(0);
    }
  }

  /**
   * Metode for  prve  gjre et trekk for en spiller. Oppdaterer
   * bde spillet og hovedvinduet (grafisk representasjon)
   * @param hvilkenSpiller Spilleren som gjr trekket
   */
  public void prvTrekk(int hvilkenSpiller){
    JTextField trekket;
    //Avgjr hvilket tekstfelt det skal leses fra
    if(hvilkenSpiller==1)
      trekket=spiller1Trekk;
    else
      trekket=spiller2Trekk;

    try{
      int rekke=-1,kolonne=-1;
      //les inn hvilklen kolonne
      kolonne=Integer.parseInt(trekket.getText());

      //prver  gjre trekket
      rekke=spillet.gjrTrekk(kolonne,hvilkenSpiller);
      if(rekke>=0){
        //dersom trekket var gyldig, skal brikken tegnes i hovedvinduet
        ((GUISpilleBrett)venstrePanel).tegnEnBrikke(rekke,kolonne,
                spillet.hentNvrendeSpiller().hentSpillerFarge());

        //kontroller om spillet er ferdig
        if(spillet.spillErFerdig()){
          if(spillet.hentVinner()!=null){
            beskjeder.setText("Gratulerer spiller "+hvilkenSpiller+
                              ", du vant!");
            spiller1GjrTrekk.setEnabled(false);
            spiller1Trekk.setEditable(false);
            spiller2GjrTrekk.setEnabled(false);
            spiller2Trekk.setEditable(false);
          }
          else
            beskjeder.setText("Fullt spillebrett. Uavgjort.");
        }
        else{
          //hvis spillet ikke er ferdig setter vi neste spiller sin tur
          if(hvilkenSpiller==1)
            settSpillerSinTur(2);
          else
            settSpillerSinTur(1);

          beskjeder.setText("");
        }
      }
      else
        beskjeder.setText("Ugyldig trekk. Prv igjen.");
    }
    catch(Exception feil){
      beskjeder.setText("Feil i tallformat. "+ feil);
    }
    getContentPane().validate();
  }

  /**
   * Metode for  opprette et nytt spill. Oppretter bde et Spill objekt
   * og et GUISpilleBrett objekt som skal vise spillets tilstand i
   * hovedvinduet.
   * @param rekker Antall rekker det skal vre p spillebrettet
   * @param kolonner Antall kolonner det skal vre p spillebrettet
   * @param hvemStarter Hvilken spiller som skal starte spillet
   */
  public void opprettNyttSpill(int rekker, int kolonner, int hvemStarter){
    //Opprett et nytt spill objekt
    spillet=new Spill(rekker,kolonner,hvemStarter);

    //La den spilleren som skal starte spillet f mulighet til  gjre et trekk
    settSpillerSinTur(hvemStarter);

    //Opprett et spillebrett for visning i hovedvinduet
    venstrePanel=new GUISpilleBrett(rekker,kolonner);
    beskjeder.setText("");
    spiller1Trekk.setText("0");
    spiller2Trekk.setText("0");
    Container content=getContentPane();
    //fjern den komponenten som ble vist der fra fr
    content.remove(0);
    //legg til det nye spillebrettet
    content.add(venstrePanel,0);
    content.validate();
  }

  /**
   * Metode for  lese inn et spill fra fil med objekt-serialisering
   * @param filnavn Filnavnet det skal leses inn fra
   */
  public void lesSpillFraFil(String filnavn){
    try{
      ObjectInputStream objin=
        new ObjectInputStream(new FileInputStream(filnavn));
      //les inn spillet
      spillet=(Spill)objin.readObject();
      if(spillet!=null){
        //Innlesingen var vellykket, oppdater GUI etter spillets tilstand
        IBrikke [][]spilleBrettet=spillet.hentSpilleBrett().hentSpilleBrett();
        int rekker=spilleBrettet.length;
        int kolonner=spilleBrettet[0].length;
        int hvilkenSpillerSinTur=((Spill)spillet).hvilkenSpillerSinTur();
        settSpillerSinTur(hvilkenSpillerSinTur);

        venstrePanel=new GUISpilleBrett(rekker,kolonner);
        Container content=getContentPane();
        content.remove(0);
        content.add(venstrePanel,0);
        content.validate();
        //tegn hver brikke p spillebrettet
        for(int i=0;i<rekker;i++)
          for(int j=0;j<kolonner;j++)
            if(spilleBrettet[i][j]!=null)
              ((GUISpilleBrett)venstrePanel).
                  tegnEnBrikke(i,j,
                      spilleBrettet[i][j].hentSpiller().hentSpillerFarge());
      }
    }
    catch(Exception feil){
      JOptionPane.showMessageDialog(this,"Lesingen fra fil var mislykket! " +
                                          feil);
    }
  }

  /**
   * Metode for  skrive et spill til fil med objekt-serialisering
   * @param filnavn Filnavnet det skal skrives til
   */
  public void skrivSpillTilFil(String filnavn){
    try{
      ObjectOutputStream objout=
        new ObjectOutputStream(new FileOutputStream(filnavn));
      objout.writeObject(spillet);
    }
    catch(Exception feil){
      JOptionPane.showMessageDialog(this,"Lagringen til fil var mislykket! " +
                                          feil);
    }
  }

  /**
   * Metode for  sette hvilken spiller som skal ha mulighet til  gjre et
   * trekk i hovedvinduet.
   * @param hvilkenSpiller Hvilken spiller som skal gjre neste trekk
   */
  public void settSpillerSinTur(int hvilkenSpiller){
    if(hvilkenSpiller==1){
      //Gir spiller 1 mulighet til  skrive inn trekk og trykke p sin knapp
      spiller1GjrTrekk.setEnabled(true);
      spiller1Trekk.setEditable(true);
      //Hindrer spiller 2 i  gjre noe
      spiller2GjrTrekk.setEnabled(false);//Umulig  trykke p knappen
      spiller2Trekk.setEditable(false);//Umulig  skrive inn tekst
    }
    else{
      spiller1GjrTrekk.setEnabled(false);
      spiller1Trekk.setEditable(false);
      spiller2GjrTrekk.setEnabled(true);
      spiller2Trekk.setEditable(true);
    }
  }

}
















