import java.io.Serializable;

/**
 * Klassen SpilleBrett implementerer interfacen (kontrakten) ISPilleBrett.
 * Klassen representerer et spillebrett for spillet Fire-P-Rad.
 * @author Trond Hellem B
 */
public class SpilleBrett implements ISpilleBrett,Serializable{

  private IBrikke [][] spilleBrettet;//array som representerer spillebrettet
  private int rekker,kolonner;//antall rekker og kolonner p spillebrettet

  /**
   * Konstruktr for objekter av klassen SpilleBrett.
   * Oppretter et spillebrett av spesifisert strrelse
   * @param antallRekker Antall rekker spillebrettet skal ha
   * @param antallKolonner Antall kolonner spillebrettet skal ha.
   */
  public SpilleBrett(int antallRekker,int  antallKolonner){

    //Oppretter spillebrettet
    spilleBrettet=new Brikke[antallRekker][antallKolonner];

    rekker=antallRekker;
    kolonner=antallKolonner;
  }

  /**
   * Metode for  f tilgang til brikkene p selve spillebrettet
   * @return IBrikke[][] Spillebrettet som en IBrikke array
   */
  public IBrikke[][] hentSpilleBrett(){
    return spilleBrettet;
  }

  /**
   * Metode som sjekker om et trekk kan gjres i det spesifiserte
   * kolonnenummeret
   * @return boolean True dersom den spesifiserte kolonnen ikke er full,
   * ellers false (dersom full kolonne eller ugyldig kolonnenummer)
   */
  public boolean gyldigKolonneNummer(int kolonne){
    return kolonne>=0 && kolonne<kolonner &&
           spilleBrettet[rekker-1][kolonne]==null;
  }

  /**
   * Setter inn en brikke p brettet for den spesifiserte spilleren og
   * kolonne. Fargen p brikken bestemmes av spillerens farge.
   * @return IBrikke Brikken som ble lagt til brettet.
   */
  public IBrikke leggTilBrikke(int kolonne, ISpiller spiller){
    Brikke nyBrikke = null;//referanse til brikken som skal returneres
    if(gyldigKolonneNummer(kolonne)){
      //Trekket kan utfres

      int rekke=-1;

      //Finner rekken brikken skal plasseres i
      for(int i=0;i<rekker && rekke<0;i++)
        if(spilleBrettet[i][kolonne]==null)
          rekke=i;

      //Oppretter en ny brikke og legger den p spillebrettet
      nyBrikke=new Brikke(rekke,kolonne,spiller);
      spilleBrettet[rekke][kolonne]=nyBrikke;
    }
    return nyBrikke;
  }

  /**
   * Metode for  f vite hvor mange kolonner det er p spillebrettet
   * @return int Antall kolonner p spillebrettet
   */
  public int hentAntallKolonner(){
    return kolonner;
  }

  /**
   * Metode for  f vite hvor mange rekker det er p spillebrettet
   * @return int Antall rekker p spillebrettet
   */
  public int hentAntallRekker(){
    return rekker;
  }
}